﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using Telerik.Windows.Controls;
using Telerik.Windows.Documents.Model;
using Telerik.Windows.Documents.FormatProviders.Html;
using Telerik.Windows.Documents.FormatProviders;
using System.IO;
using Telerik.Windows.Documents.UI;

namespace RadGridView_SL5_AR_1
{
    public enum ExportFormat
    {
        Excel,
        Csv
    }

    public class ExportSettings
    {
        public ExportFormat Format
        {
            get;
            set;
        }
    }

    public static class PrintAndExportExtensions
    {
        public static void Export(this RadGridView grid, ExportSettings settings)
        {
            var dialog = new SaveFileDialog();

            if (settings.Format == ExportFormat.Excel)
            {
                dialog.DefaultExt = "*.xls";
                dialog.Filter = "Excel Workbook (*.xls)|*.xls";
            }

            else if (settings.Format == ExportFormat.Csv)
            {
                dialog.DefaultExt = "*.csv";
                dialog.Filter = "CSV (Comma delimited) (*.csv)|*.csv";
            }

            if (dialog.ShowDialog() == true)
            {
                if (settings.Format == ExportFormat.Csv)
                {
                    using (var output = dialog.OpenFile())
                    {
                        grid.Export(output, new GridViewExportOptions()
                        {
                            Format = Telerik.Windows.Controls.ExportFormat.Csv,
                            ShowColumnFooters = grid.ShowColumnFooters,
                            ShowColumnHeaders = grid.ShowColumnHeaders,
                            ShowGroupFooters = grid.ShowGroupFooters
                        });
                    }
                }
                else
                {
                    var document = CreateDocument(grid);

                    if (document != null)
                    {
                        document.LayoutMode = DocumentLayoutMode.Paged;
                        document.Measure(RadDocument.MAX_DOCUMENT_SIZE);
                        document.Arrange(new RectangleF(PointF.Empty, document.DesiredSize));

                        IDocumentFormatProvider provider = null;

                        if (settings.Format == ExportFormat.Excel)
                        {
                            provider = new HtmlFormatProvider();
                        }

                        using (var output = dialog.OpenFile())
                        {
                            provider.Export(document, output);
                        }
                    }
                }
            }
        }

        public static void Print(this RadGridView grid)
        {
            var rtb = CreateRadRichTextBox(grid);
            var window = new RadWindow() { Height = 0, Width = 0, Opacity = 0, Content = rtb };
            rtb.PrintCompleted += (s, e) => { window.Close(); };
            window.Show();

            rtb.Print("MyDocument", Telerik.Windows.Documents.UI.PrintMode.Native);
        }

        public static void PrintPreview(this RadGridView grid)
        {
            var rtb = CreateRadRichTextBox(grid);
            var window = CreatePreviewWindow(rtb);
            window.ShowDialog();
        }

        private static RadWindow CreatePreviewWindow(RadRichTextBox rtb)
        {
            var printButton = new RadButton()
            {
                Content = "Print",
                Margin = new Thickness(10, 0, 10, 0),
                FontWeight = FontWeights.Bold,
                Width = 80
            };

            printButton.Click += (s, e) =>
            {
                rtb.Print("MyDocument", Telerik.Windows.Documents.UI.PrintMode.Native);
            };

            var sp = new StackPanel() { Height = 26, Orientation = Orientation.Horizontal, Margin = new Thickness(10) };

            sp.Children.Add(new TextBlock() { Text = "Orientation:", Margin = new Thickness(10, 0, 3, 0), VerticalAlignment = VerticalAlignment.Center });

            var radComboBoxPageOrientation = new RadComboBox()
            {
                ItemsSource = new string[] { "Portrait", "Landscape" },
                SelectedIndex = 0
            };
            sp.Children.Add(radComboBoxPageOrientation);

            radComboBoxPageOrientation.SelectionChanged += (s, e) =>
            {
                rtb.ChangeSectionPageOrientation((PageOrientation)Enum.Parse(typeof(PageOrientation),
                    radComboBoxPageOrientation.Items[radComboBoxPageOrientation.SelectedIndex].ToString(), true));
            };

            sp.Children.Add(new TextBlock() { Text = "Size:", Margin = new Thickness(10, 0, 3, 0), VerticalAlignment = VerticalAlignment.Center });

            var radComboBoxPageSize = new RadComboBox()
            {
                ItemsSource = new string[] { "A0", "A1", "A2", "A3", "A4", "A5", "Letter" },
                Height = 25,
                SelectedIndex = 4,
            };
            sp.Children.Add(radComboBoxPageSize);

            radComboBoxPageSize.SelectionChanged += (s, e) =>
            {
                rtb.ChangeSectionPageSize(PaperTypeConverter.ToSize((PaperTypes)Enum.Parse(typeof(PaperTypes),
                    radComboBoxPageSize.Items[radComboBoxPageSize.SelectedIndex].ToString(), true)));
            };

            sp.Children.Add(printButton);

            var g = new Grid();
            g.RowDefinitions.Add(new RowDefinition() { Height = GridLength.Auto });
            g.RowDefinitions.Add(new RowDefinition());
            g.Children.Add(sp);
            g.Children.Add(rtb);

            Grid.SetRow(rtb, 1);

            return new RadWindow()
            {
                Content = g,
                Width = 900,
                Height = 600,
                Header = "Print Preview",
                WindowStartupLocation = Telerik.Windows.Controls.WindowStartupLocation.CenterScreen

            };
        }

        private static RadRichTextBox CreateRadRichTextBox(RadGridView grid)
        {
            return new RadRichTextBox()
            {
                IsReadOnly = true,
                LayoutMode = DocumentLayoutMode.Paged,
                IsSelectionEnabled = false,
                IsSpellCheckingEnabled = false,
                Document = CreateDocument(grid)
            };
        }

        private static RadDocument CreateDocument(RadGridView grid)
        {
            RadDocument document = null;

            using (var stream = new MemoryStream())
            {
                grid.Export(stream, new GridViewExportOptions()
                {
                    Format = Telerik.Windows.Controls.ExportFormat.Html,
                    ShowColumnFooters = grid.ShowColumnFooters,
                    ShowColumnHeaders = grid.ShowColumnHeaders,
                    ShowGroupFooters = grid.ShowGroupFooters
                });
                stream.Position = 0;

                document = new HtmlFormatProvider().Import(stream);
            }

            return document;
        }
    }
}
